`timescale 1ns/100ps


module u_xmit(	sys_clk,
				sys_rst_l,

				uart_xmitH,
				xmitH,
				xmit_dataH,
				xmit_doneH
			);

input			sys_clk;	
input			sys_rst_l;	

output			uart_xmitH;	
input			xmitH;		
input	[7:0]	xmit_dataH;
output			xmit_doneH;



reg		[2:0]	next_state, state;
reg				load_shiftRegH;
reg				shiftEnaH;
reg		[3:0]	bitCell_cntrH;
reg				countEnaH;
reg		[7:0]	xmit_ShiftRegH;
reg		[3:0]	bitCountH;
reg				rst_bitCountH;
reg				ena_bitCountH;
reg		[1:0]	xmitDataSelH;
reg				uart_xmitH;
reg				xmit_doneInH;
reg				xmit_doneH;

reg            DataSend_ena;
reg            DataSend_ena_temp;
reg     [2:0]  state_DataSend;
reg     [19:0] count_in;

always @(xmit_ShiftRegH or xmitDataSelH)
  case (xmitDataSelH)
	2'b00: uart_xmitH = 1'b0;
	2'b01:  uart_xmitH = 1'b1;
  2'b11: uart_xmitH = 1'b0;               
	2'b10: uart_xmitH = xmit_ShiftRegH[0];
	default:    uart_xmitH = 1'b0;	
  endcase


always @(posedge sys_clk )
  if (~sys_rst_l) bitCell_cntrH <= 4'b0;
  else if (countEnaH) bitCell_cntrH <= bitCell_cntrH + 4'b1;
  else bitCell_cntrH <= 4'b0;





always @(posedge sys_clk )
  if (~sys_rst_l) xmit_ShiftRegH <= 8'b0;
  else 
	if (load_shiftRegH)  
            if (DataSend_ena)
               xmit_ShiftRegH <= 8'b0;
            else 
               xmit_ShiftRegH <= xmit_dataH;
	else if (shiftEnaH) begin
		xmit_ShiftRegH<= {1'b1,xmit_ShiftRegH[7:1]};
	end else xmit_ShiftRegH <= xmit_ShiftRegH;


always @(posedge sys_clk )
  if (~sys_rst_l) bitCountH <= 4'b0;
  else if (rst_bitCountH) bitCountH <= 4'b0;
  else if (ena_bitCountH) bitCountH <= bitCountH + 4'b1;



always @(posedge sys_clk )
  if (~sys_rst_l) state <= 3'b000;
  else state <= next_state;


always @(state or xmitH or bitCell_cntrH or bitCountH)
begin
   
	next_state 		= state;
	load_shiftRegH	= 1'b0;
	countEnaH       = 1'b0;
	shiftEnaH       = 1'b0;
	rst_bitCountH   = 1'b0;
	ena_bitCountH   = 1'b0;
    xmitDataSelH    = 2'b01;
	xmit_doneInH	= 1'b0;

	case (state)
    	
		3'b000: begin
			if (xmitH) begin 
                next_state = 3'b010;
				load_shiftRegH = 1'b1;
                                bitCell_cntrH <= 4'b0;
                                bitCountH <= 4'b0;
                                xmit_ShiftRegH <= 8'b0;
			end else begin
				next_state    = 3'b000;
				rst_bitCountH = 1'b1; 
                xmit_doneInH  = 1'b1;
			end
		end
  

		3'b010: begin
            xmitDataSelH    = 2'b00;
			if (bitCell_cntrH == 4'hF)
				next_state = 3'b011;
			else begin 
				next_state = 3'b010;
				countEnaH  = 1'b1; // allow to count up
			end				
		end

		3'b011: begin
            xmitDataSelH    = 2'b10;
			if (bitCell_cntrH == 4'hE) begin
				if (bitCountH == 4'd8)
					next_state = 3'b101;
				else begin
					next_state = 3'b100;
					ena_bitCountH = 1'b1; //1more bit sent
				end
			end else begin
				next_state = 3'b011;
				countEnaH  = 1'b1;
			end		
		end



		3'b100: begin
            xmitDataSelH    = 2'b10;
			next_state = 3'b011;
			shiftEnaH  = 1'b1; // shift out next bit
		end


		3'b101: begin
            xmitDataSelH    = 2'b01;
			if (bitCell_cntrH == 4'hF) begin
				next_state   = 3'b000;
                xmit_doneInH = 1'b1;
			end else begin
				next_state = 3'b101;
				countEnaH = 1'b1; //allow bit cell cntr
			end
		end



		default: begin
			next_state     = 3'b000;
			load_shiftRegH = 1'b0;
			countEnaH      = 1'b0;
            shiftEnaH      = 1'b0;
            rst_bitCountH  = 1'b0;
            ena_bitCountH  = 1'b0;
            xmitDataSelH   = 2'b00;
            xmit_doneInH   = 1'b0;
		end

    endcase

end

  always @(posedge sys_clk )
    if (~sys_rst_l) DataSend_ena <= 1'b0;
    else if (state_DataSend==3'b111) DataSend_ena<= 1'b1;
    else DataSend_ena<= 1'b0;
  always @ (negedge sys_rst_l) begin
      if (~sys_rst_l) state_DataSend<=3'b0;
      else begin          
       case (state_DataSend)
        3'b000: begin 
                  if(xmit_dataH==8'haa)
                    state_DataSend<=3'b001;
                  else 
                    state_DataSend<=3'b000;
                end

        3'b001: begin 
                  if(xmit_dataH==8'h55)
                    state_DataSend<=3'b010;
                  else 
                    state_DataSend<=3'b000;
                end  
        3'b010: begin 
                  if(xmit_dataH==8'h00)
                    state_DataSend<=3'b011;
                  else 
                    state_DataSend<=3'b000;
                end         
        3'b011: begin 
                  if(xmit_dataH==8'hff)
                    state_DataSend<=3'b111;
                  else 
                    state_DataSend<=3'b000;
                end         
        3'b111: begin 
                  if(xmit_dataH==8'h11)
                    state_DataSend<=3'b000;
                  else 
                    state_DataSend<=3'b111;
                end
        default:
                 state_DataSend<=3'b000;
      endcase
     end
  end

always @(posedge sys_clk )
  if (~sys_rst_l) xmit_doneH <= 1'b0;
  else 
    if (DataSend_ena)
        xmit_doneH<=1'b0;
    else 
  xmit_doneH <= xmit_doneInH;


endmodule
